"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isInvalid = exports.isAtRule = exports.isDeclaration = exports.isRule = exports.getAstNodeAt = void 0;
const css_selector_parser_1 = require("@tokey/css-selector-parser");
const CSSValue = __importStar(require("@tokey/css-value-parser"));
const index_internal_1 = require("@stylable/core/dist/index-internal");
const edit_time_parser_1 = require("./edit-time-parser");
// ToDo: move to tokey
function walkValue(valueList, visit) {
    for (const node of valueList) {
        const changeWalk = visit(node);
        if (changeWalk === css_selector_parser_1.walk.stopAll) {
            return;
        }
        if (node.type === 'call' && changeWalk !== css_selector_parser_1.walk.skipNested) {
            walkValue(node.args, visit);
        }
    }
}
function isClosed(node) {
    var _a;
    const isLast = node.parent && node.parent.nodes[node.parent.nodes.length - 1] === node;
    if (node.type === 'decl') {
        return isLast ? (_a = node.parent) === null || _a === void 0 ? void 0 : _a.raws.semicolon : true;
    }
    return true;
}
function getAstNodeAt(parseData, targetOffset) {
    const result = {
        base: {
            type: 'base',
            node: parseData.ast,
            offsetInNode: targetOffset,
            where: 'root',
        },
        selector: undefined,
        declValue: undefined,
        atRuleParams: undefined,
    };
    parseData.ast.walk((node) => {
        const { isInRange: inNode, nodeEndsAfterCaret, nodeStartBeforeCaret, } = isPostcssNodeInRange(node, targetOffset);
        // check for space after unclosed node
        let afterNodeContent = false;
        if (!inNode && !nodeEndsAfterCaret && !isClosed(node)) {
            afterNodeContent = isPostcssNodeInRange(node.parent, targetOffset).isInRange;
        }
        if (!inNode && !afterNodeContent) {
            // not part of node: bailout completely if node is after caret
            return nodeStartBeforeCaret ? undefined : false;
        }
        const baseNodeOffset = node.source.start.offset;
        result.base.node = node;
        result.base.offsetInNode = targetOffset - baseNodeOffset;
        if (node.type === 'comment') {
            result.base.where = 'comment';
            return false;
        }
        else if (isInvalid(node)) {
            result.base.where = 'invalid';
        }
        const checkContext = {
            baseNodeOffset,
            targetOffset,
            afterNodeContent,
            result,
        };
        // check nested structures
        checkRuleSelector(node, checkContext, parseData);
        checkDeclValue(node, checkContext);
        checkAtRuleParams(node, checkContext);
        return;
    });
    return result;
}
exports.getAstNodeAt = getAstNodeAt;
function checkRuleSelector(node, { baseNodeOffset, targetOffset, result }, parseData) {
    var _a, _b;
    const isInRule = isRule(node);
    if (isInRule ||
        isInvalid(node) ||
        (isDeclaration(node) &&
            ((_a = parseData.ambiguousNodes
                .get(node)) === null || _a === void 0 ? void 0 : _a.find((type) => type === edit_time_parser_1.AMBIGUITY.POSSIBLE_UNOPENED_RULE))) ||
        (isAtRule(node) && node.name === 'st-scope')) {
        let selector = '';
        let selectorAfterWhiteSpaceLength = 0;
        let selectorStartOffset = baseNodeOffset;
        if (node.type === 'rule') {
            selector = node.selector;
            selectorAfterWhiteSpaceLength = ((_b = node.raws.between) === null || _b === void 0 ? void 0 : _b.length) || 0;
        }
        else if (isDeclaration(node)) {
            selector = node.prop + node.raws.between + node.value; // + node.important;
        }
        else if (isAtRule(node)) {
            selector = node.params;
            selectorStartOffset += '@st-scope'.length + 1;
        }
        else {
            selector = node.value;
        }
        const selectorEnd = selectorStartOffset + selector.length;
        const afterSelector = selectorEnd < targetOffset;
        const beforeBody = afterSelector && selectorEnd + selectorAfterWhiteSpaceLength >= targetOffset;
        if (afterSelector && !beforeBody) {
            // not in selector
            isInRule && (result.base.where = 'ruleBody');
            return;
        }
        const selectors = (0, index_internal_1.parseSelectorWithCache)(selector);
        result.selector = {
            type: 'selector',
            /* If there are actual selectors then Initiate with last selector that is either overridden
                by actual selector at caret, or used in case caret is in the whitespace after the selector.
               Else if there is no selector then set an empty selector to show a zero size selector.
            */
            node: selectors && selectors.length
                ? selectors[selectors.length - 1]
                : {
                    type: 'selector',
                    after: '',
                    before: '',
                    nodes: [],
                    start: 0,
                    end: 0,
                },
            offsetInNode: !afterSelector ? 0 : selector.length + targetOffset - selectorEnd,
            afterSelector,
            parents: [],
        };
        isInRule && (result.base.where = 'ruleBetweenSelectorAndBody');
        if (selectors.length && !afterSelector) {
            isInRule && (result.base.where = 'ruleSelector');
            const selectorTargetOffset = targetOffset - selectorStartOffset;
            (0, css_selector_parser_1.walk)(selectors, (selectorNode, _index, _nodes, parents) => {
                const isTargetAfterStart = selectorStartOffset + selectorNode.start < targetOffset;
                const isTargetBeforeEnd = selectorStartOffset + selectorNode.end >= targetOffset;
                const isSelector = selectorNode.type === 'selector';
                if (!isTargetAfterStart) {
                    // selector is after the target offset
                    if (isSelector && selectorStartOffset + selectorNode.start === targetOffset) {
                        // start of selector
                        result.selector.node = selectorNode;
                    }
                    return css_selector_parser_1.walk.stopAll;
                }
                else if (!isTargetBeforeEnd) {
                    // selector ends before target offset
                    return css_selector_parser_1.walk.skipNested;
                }
                result.selector.parents = [...parents];
                result.selector.node = selectorNode;
                result.selector.offsetInNode = selectorTargetOffset - selectorNode.start;
                if (!isSelector) {
                    result.selector.afterSelector = false;
                }
                else if (selectorStartOffset + selectorNode.start - selectorNode.after.length <
                    targetOffset) {
                    result.selector.afterSelector = true;
                }
                return;
            });
        }
        result.selector.parents = [node, ...result.selector.parents];
    }
}
function checkDeclValue(node, checkContext) {
    if (isDeclaration(node)) {
        const between = node.raws.between;
        const valueStart = checkContext.baseNodeOffset + node.prop.length + between.length;
        const valueEnd = valueStart + node.value.length;
        const isInValue = checkValue({
            type: 'declValue',
            value: node.value,
            node,
            valueStart,
            valueEnd,
            afterSpace: 0,
            checkContext,
        });
        let where = 'declValue';
        const base = checkContext.result.base;
        if (isInValue) {
            where = 'declValue';
        }
        else if (base.offsetInNode > node.prop.length) {
            const spaceAfterColon = between.length - between.indexOf(':');
            if (valueStart - spaceAfterColon >= checkContext.targetOffset) {
                where = 'declBetweenPropAndColon';
            }
            else {
                where = 'declBetweenColonAndValue';
            }
        }
        else {
            where = 'declProp';
        }
        base.where = where;
    }
}
function checkAtRuleParams(node, checkContext) {
    if (isAtRule(node)) {
        const valueStart = checkContext.baseNodeOffset + 1 + node.name.length + node.raws.afterName.length;
        const valueEnd = valueStart + node.params.length;
        const isInParams = checkValue({
            type: 'atRuleParams',
            value: node.params,
            node,
            valueStart,
            valueEnd,
            afterSpace: node.raws.between.length,
            checkContext,
        });
        let where = 'declValue';
        const base = checkContext.result.base;
        if (isInParams) {
            where = 'atRuleParams';
        }
        else if (checkContext.baseNodeOffset + node.name.length + 1 >=
            checkContext.targetOffset) {
            where = 'atRuleName';
        }
        else {
            where = 'atRuleBody';
        }
        base.where = where;
    }
}
function checkValue({ value, node, type, valueStart, valueEnd, afterSpace, checkContext: { targetOffset, result, afterNodeContent }, }) {
    const isAfterValue = valueEnd < targetOffset;
    const isInIncludedSpace = afterNodeContent || (isAfterValue && valueEnd + afterSpace >= targetOffset);
    if (valueStart > targetOffset || (isAfterValue && !isInIncludedSpace)) {
        // not in value
        return false;
    }
    const ast = CSSValue.parseCSSValue(value);
    const valueLocation = {
        type,
        node: ast,
        offsetInNode: !isInIncludedSpace ? 0 : value.length + targetOffset - valueEnd,
        parents: [node],
        afterValue: isInIncludedSpace,
    };
    if (!isInIncludedSpace) {
        walkValue(ast, (node) => {
            const isTargetAfterStart = valueStart + node.start < targetOffset ||
                (valueStart === targetOffset && node.start === 0);
            const isTargetBeforeEnd = valueStart + node.end >= targetOffset;
            if (!isTargetAfterStart) {
                // value is after the target offset
                return css_selector_parser_1.walk.stopAll;
            }
            else if (!isTargetBeforeEnd) {
                // value ends before target offset
                return css_selector_parser_1.walk.skipNested;
            }
            // update
            if (!Array.isArray(valueLocation.node)) {
                valueLocation.parents.push(valueLocation.node);
            }
            valueLocation.node = node;
            valueLocation.offsetInNode = targetOffset - valueStart - node.start;
            return;
        });
    }
    result[type] = valueLocation; // ToDo: figure out type issue
    return true;
}
function isPostcssNodeInRange(node, target) {
    var _a, _b;
    const result = {
        isInRange: false,
        nodeStartBeforeCaret: false,
        nodeEndsAfterCaret: false,
    };
    if (((_a = node.source) === null || _a === void 0 ? void 0 : _a.start) && ((_b = node.source) === null || _b === void 0 ? void 0 : _b.end)) {
        const beforeSize = node.type === 'rule' ? node.raws.before.length : 0;
        result.nodeStartBeforeCaret = node.source.start.offset - beforeSize <= target;
        result.nodeEndsAfterCaret = node.source.end.offset + 1 >= target;
        result.isInRange = result.nodeStartBeforeCaret && result.nodeEndsAfterCaret;
    }
    return result;
}
function isRule(node) {
    return (node === null || node === void 0 ? void 0 : node.type) === `rule`;
}
exports.isRule = isRule;
function isDeclaration(node) {
    return (node === null || node === void 0 ? void 0 : node.type) === `decl`;
}
exports.isDeclaration = isDeclaration;
function isAtRule(node) {
    return (node === null || node === void 0 ? void 0 : node.type) === `atrule`;
}
exports.isAtRule = isAtRule;
function isInvalid(node) {
    return (node === null || node === void 0 ? void 0 : node.type) === `invalid`;
}
exports.isInvalid = isInvalid;
//# sourceMappingURL=ast-from-position.js.map