/// <reference types="externals" />
import { Invalid } from './invalid-node';
import Parser from 'postcss/lib/parser';
import * as postcss from 'postcss';
type PostcssToken = [string, string, number, number];
type AnyNode = Invalid | postcss.AnyNode | postcss.Container | postcss.Document | postcss.Declaration;
export interface ParseForEditingResult {
    ast: EditTimeParser['root'];
    errorNodes: EditTimeParser['errorNodes'];
    ambiguousNodes: EditTimeParser['ambiguousNodes'];
}
export declare function parseForEditing(source: string, { from }?: {
    from?: string;
}): {
    ast: postcss.Root;
    errorNodes: Map<AnyNode, string[]>;
    ambiguousNodes: Map<AnyNode, string[]>;
};
export type ParseReport = string;
export declare const ERRORS: Record<string, ParseReport>;
export declare const AMBIGUITY: Record<string, ParseReport>;
declare class EditTimeParser extends Parser {
    errorNodes: Map<AnyNode, string[]>;
    ambiguousNodes: Map<AnyNode, string[]>;
    constructor(input: postcss.Input);
    createTokenizer(): void;
    private reportNode;
    private createInvalidNode;
    private getLastOffset;
    private extendParentEnd;
    private unOpenedRule;
    private unOpenedRuleOrDecl;
    init(node: postcss.Node, offset: number): void;
    unknownWord(): void;
    unexpectedClose(token: PostcssToken): void;
    unclosedBlock(): void;
    doubleColon(): void;
    unnamedAtrule(node: postcss.AtRule): void;
    checkMissedSemicolon(): void;
    other(start: PostcssToken): void;
}
export {};
//# sourceMappingURL=edit-time-parser.d.ts.map