"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompletions = void 0;
const index_internal_1 = require("@stylable/core/dist/index-internal");
const node_path_1 = __importDefault(require("node:path"));
const completion_types_1 = require("../../lib/completion-types");
function getCompletions(context) {
    var _a;
    const completions = [];
    const selectorContext = context.getSelectorContext();
    if (!selectorContext) {
        return completions;
    }
    const { resolvedSelectorChain = [], nodeAtCursor = null, fullSelectorAtCursor = '', selectorAtCursor = '', } = selectorContext;
    if (selectorAtCursor === '::') {
        // No state completion directly after pseudo_element opener: bailout
        return completions;
    }
    const current = resolvedSelectorChain[resolvedSelectorChain.length - 1] || {
        // default to root
        resolved: context.stylable.resolver.resolveExtends(context.meta, 'root'),
    };
    if (current === null || current === void 0 ? void 0 : current.resolved) {
        const pos = context.getPosition();
        const states = collectStates(current.resolved);
        const isInStateParen = fullSelectorAtCursor.match(/:(.+)\((\w*)$/);
        const inStateParenDef = isInStateParen && states[isInStateParen === null || isInStateParen === void 0 ? void 0 : isInStateParen[1]];
        const enumParamDef = getEnumParamDef(inStateParenDef === null || inStateParenDef === void 0 ? void 0 : inStateParenDef.def);
        if (enumParamDef) {
            // complete enum options
            const currentParam = isInStateParen[2];
            const completeFullSelector = currentParam === '';
            for (const option of enumParamDef.arguments) {
                if (typeof option !== 'string') {
                    continue;
                }
                let startDelta = 0;
                if (option.startsWith(currentParam)) {
                    // partial completion
                    startDelta = currentParam.length;
                }
                else if (!completeFullSelector) {
                    continue;
                }
                completions.push((0, completion_types_1.stateEnumCompletion)(option, normalizeDefinitionSheetLocation(context.meta, inStateParenDef.meta), (0, completion_types_1.range)(pos, { deltaStart: -startDelta })));
            }
        }
        else {
            // complete custom states
            const completeFullSelector = suggestCompleteStates(nodeAtCursor, selectorAtCursor, states, current.resolved);
            for (const [name, { def, meta }] of Object.entries(states)) {
                const isAlreadyUsed = (_a = current['pseudo_class']) === null || _a === void 0 ? void 0 : _a.find((exist) => exist.value === name);
                if (isAlreadyUsed) {
                    continue;
                }
                const stateSelector = ':' + name;
                let startDelta = 0;
                if (stateSelector.startsWith(selectorAtCursor)) {
                    // partial completion
                    startDelta = selectorAtCursor.length;
                }
                else if (!completeFullSelector) {
                    continue;
                }
                const stateWithParam = !!(def && typeof def === 'object');
                const stateType = stateWithParam ? def.type : null;
                const originFile = normalizeDefinitionSheetLocation(context.meta, meta);
                completions.push((0, completion_types_1.stateCompletion)(name, originFile, (0, completion_types_1.range)(pos, { deltaStart: -startDelta }), stateType, stateWithParam));
            }
        }
    }
    return completions;
}
exports.getCompletions = getCompletions;
function getEnumParamDef(stateDef) {
    if (!stateDef || typeof stateDef !== 'object') {
        return;
    }
    else if (stateDef.type === 'enum') {
        return stateDef;
    }
    else if (index_internal_1.STCustomState.isTemplateState(stateDef) && stateDef.params[0].type === 'enum') {
        return stateDef.params[0];
    }
    return;
}
function suggestCompleteStates(nodeAtCursor, selectorAtCursor, states, resolveElement) {
    return (
    // prev is not a pseudo class
    (nodeAtCursor === null || nodeAtCursor === void 0 ? void 0 : nodeAtCursor.type) !== 'pseudo_class' ||
        // prev is a known native pseudo class
        index_internal_1.nativePseudoClasses.includes(selectorAtCursor) ||
        // prev is a known custom state
        !!states[selectorAtCursor.slice(1)] ||
        // prev is a known custom selector
        !!(resolveElement[0] &&
            index_internal_1.STCustomSelector.getCustomSelector(resolveElement[0].meta, selectorAtCursor.slice(3))));
}
function normalizeDefinitionSheetLocation(originMeta, defMeta) {
    if (defMeta === originMeta) {
        return 'Local file';
    }
    let relPath = node_path_1.default.relative(node_path_1.default.dirname(originMeta.source), defMeta.source);
    if (!relPath.startsWith('.')) {
        relPath = './' + relPath;
    }
    return relPath;
}
function collectStates(resolveChain) {
    return resolveChain.reduce((acc, cur) => {
        const symbol = cur.symbol;
        if (symbol._kind === 'class') {
            const symbolStates = symbol[`-st-states`];
            if (symbolStates) {
                Object.keys(symbolStates).forEach((k) => {
                    const symbolStates = symbol[`-st-states`];
                    if (symbolStates && symbolStates[k] !== undefined) {
                        acc[k] = {
                            def: symbolStates[k],
                            meta: cur.meta,
                        };
                    }
                });
            }
        }
        return acc;
    }, {});
}
//# sourceMappingURL=ls-css-pseudo-class.js.map