import { TextDocument } from 'vscode-css-languageservice';
import { CSSResolve, Stylable, StylableMeta } from '@stylable/core';
import type { StylableFile } from '../lib/service';
import { getAstNodeAt } from './ast-from-position';
import { ParseReport } from './edit-time-parser';
import { ImmutableSelectorNode } from '@tokey/css-selector-parser';
type ResolveChainItem = {
    [Type in ImmutableSelectorNode['type']]?: Extract<ImmutableSelectorNode, {
        type: Type;
    }>[];
} & {
    resolved?: CSSResolve[];
    resolvedNode?: ImmutableSelectorNode;
};
export declare class LangServiceContext {
    stylable: Stylable;
    private fileData;
    private offset;
    meta: StylableMeta;
    errorNodes: Map<any, ParseReport[]>;
    ambiguousNodes: Map<any, ParseReport[]>;
    location: ReturnType<typeof getAstNodeAt>;
    document: TextDocument;
    constructor(stylable: Stylable, fileData: StylableFile, offset: number);
    getPosition(offset?: number): import("vscode-languageserver-textdocument").Position;
    isInRoot(): boolean;
    isInSelectorAllowedSpace(): boolean;
    isInSelector(): boolean;
    getSelectorContext(): {
        nodeAtCursor: ImmutableSelectorNode | undefined;
        selectorAtCursor: string;
        fullSelectorAtCursor: string;
        selectorAstResolveMap: Map<ImmutableSelectorNode, CSSResolve<import("@stylable/core").StylableSymbol>[]>;
        nestedSelectors: (readonly import("@tokey/css-selector-parser/dist/types").ImmutableMap<import("@tokey/css-selector-parser").Selector>[])[];
        resolvedSelectorChain: ResolveChainItem[];
    } | null;
    /**
     * Find the selector that the caret is at and traverse back to collect
     * resolved elements with any selector information after them.
     */
    private aggregateResolvedChain;
    private getSelectorString;
    private collectSelector;
}
export {};
//# sourceMappingURL=lang-service-context.d.ts.map