"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LangServiceContext = void 0;
const vscode_css_languageservice_1 = require("vscode-css-languageservice");
const core_1 = require("@stylable/core");
const ast_from_position_1 = require("./ast-from-position");
const edit_time_parser_1 = require("./edit-time-parser");
const index_internal_1 = require("@stylable/core/dist/index-internal");
const vscode_uri_1 = require("vscode-uri");
const css_selector_parser_1 = require("@tokey/css-selector-parser");
const postcss = __importStar(require("postcss"));
class LangServiceContext {
    constructor(stylable, fileData, offset) {
        this.stylable = stylable;
        this.fileData = fileData;
        this.offset = offset;
        const parseResult = (0, edit_time_parser_1.parseForEditing)(fileData.content, {
            from: fileData.path,
        });
        this.errorNodes = parseResult.errorNodes;
        this.ambiguousNodes = parseResult.ambiguousNodes;
        this.location = (0, ast_from_position_1.getAstNodeAt)(parseResult, offset);
        this.meta = new index_internal_1.StylableProcessor(new core_1.Diagnostics(), this.stylable.resolveNamespace).process(parseResult.ast);
        this.document = vscode_css_languageservice_1.TextDocument.create(vscode_uri_1.URI.file(this.meta.source).toString(), 'stylable', this.fileData.stat.mtime.getTime(), this.fileData.content);
    }
    getPosition(offset = this.offset) {
        return this.document.positionAt(offset);
    }
    isInRoot() {
        return this.location.base.node.type === 'root';
    }
    isInSelectorAllowedSpace() {
        const where = this.location.base.where;
        return (this.isInSelector() ||
            where === 'root' ||
            where === 'ruleBody' ||
            where === 'atRuleBody');
    }
    isInSelector() {
        return !!this.location.selector;
    }
    getSelectorContext() {
        if (!this.isInSelectorAllowedSpace()) {
            return null;
        }
        const nestedSelectors = this.collectSelector();
        // collect the CSS resolve chain for each selector node
        const selectorAstResolveMap = new Map();
        let currentAnchor = undefined;
        for (const selector of nestedSelectors) {
            // ToDo(tech-debt): provide an internal selector resolve without the transformer
            const transformer = this.stylable.createTransformer();
            const selectorContext = transformer.createSelectorContext(this.meta, selector, // doesn't mutate due to `selectorContext.transform = false`
            postcss.rule({ selector: (0, css_selector_parser_1.stringifySelectorAst)(selector) }));
            if (currentAnchor) {
                selectorContext.currentAnchor = currentAnchor;
                selectorContext.nestingSelectorAnchor = currentAnchor;
            }
            selectorContext.transform = false;
            selectorContext.selectorAstResolveMap = selectorAstResolveMap;
            transformer.scopeSelectorAst(selectorContext);
            // ToDo: handle multiple selectors intersection
            currentAnchor = selectorContext.currentAnchor;
        }
        // reference interest points
        const locationSelector = this.location.selector;
        const nodeAtCursor = locationSelector === null || locationSelector === void 0 ? void 0 : locationSelector.node;
        const resolvedSelectorChain = this.aggregateResolvedChain(nestedSelectors, selectorAstResolveMap);
        return {
            nodeAtCursor,
            selectorAtCursor: nodeAtCursor
                ? (0, css_selector_parser_1.stringifySelectorAst)(nodeAtCursor).slice(0, this.location.selector.offsetInNode)
                : '',
            fullSelectorAtCursor: nodeAtCursor
                ? (0, css_selector_parser_1.stringifySelectorAst)(nestedSelectors[nestedSelectors.length - 1]).slice(0, nodeAtCursor.start + locationSelector.offsetInNode)
                : '',
            selectorAstResolveMap,
            nestedSelectors,
            resolvedSelectorChain,
        };
    }
    /**
     * Find the selector that the caret is at and traverse back to collect
     * resolved elements with any selector information after them.
     */
    aggregateResolvedChain(nestedSelectors, selectorAstResolveMap) {
        var _a;
        const resolvedChain = [];
        let item = {};
        const nestingSelectors = [...nestedSelectors];
        const addToItem = (node) => {
            var _a;
            var _b;
            const resolved = selectorAstResolveMap.get(node);
            if (resolved) {
                item.resolved = resolved;
                item.resolvedNode = node;
                resolvedChain.unshift(item);
                item = {};
            }
            else if (node.type) {
                (_a = item[_b = node.type]) !== null && _a !== void 0 ? _a : (item[_b] = []);
                item[node.type].push(node);
            }
        };
        const collectBack = (node, parents) => {
            if (node) {
                addToItem(node);
            }
            if (!parents.length) {
                const upperNestingSelectors = getLastWhile(nestingSelectors, (last) => !!last.length);
                if (upperNestingSelectors) {
                    // ToDo: handle multiple selector intersection type
                    const firstSelector = upperNestingSelectors[0];
                    parents.push(firstSelector);
                }
                else {
                    return;
                }
            }
            const parent = parents.pop();
            if ('nodes' in parent && parent.nodes) {
                const nodes = parent.nodes;
                let fromIndex = nodes.indexOf(node);
                fromIndex = fromIndex === -1 ? nodes.length : fromIndex;
                for (let i = fromIndex - 1; i >= 0; i--) {
                    addToItem(nodes[i]);
                }
            }
            collectBack(parent, parents);
        };
        // find node at cursor and collect back resolved location
        if (this.location.selector) {
            const caretSelector = nestingSelectors.pop();
            const targetSelectorNode = (_a = this.location.selector) === null || _a === void 0 ? void 0 : _a.node;
            (0, css_selector_parser_1.walk)(caretSelector, (node, _index, _nodes, parents) => {
                if (node === targetSelectorNode) {
                    collectBack(node, parents);
                    return css_selector_parser_1.walk.stopAll;
                }
                return;
            });
        }
        else {
            // caret is not at selector (probably nested): complete from end
            collectBack(undefined, []);
        }
        return resolvedChain;
    }
    getSelectorString(node = this.location.base.node) {
        if (node.type === 'rule') {
            return node.selector;
        }
        if (node.type === 'atrule' && node.name === 'st-scope') {
            return node.params;
        }
        return null;
    }
    collectSelector() {
        const parents = collectPostcssParents(this.location.base.node);
        const results = parents
            .map((node) => {
            return (0, index_internal_1.parseSelectorWithCache)(this.getSelectorString(node) || '');
        })
            .filter((selectors) => selectors.length !== 0);
        if (this.location.selector) {
            let selectorWithCaret = this.location.selector.parents.find((node) => node.type === 'selector');
            if (!selectorWithCaret && this.location.selector.node.type === 'selector') {
                selectorWithCaret = this.location.selector.node;
            }
            results.push([selectorWithCaret]);
        }
        else {
            const selector = this.getSelectorString(this.location.base.node);
            if (selector) {
                results.push((0, index_internal_1.parseSelectorWithCache)(selector));
            }
        }
        return results;
    }
}
exports.LangServiceContext = LangServiceContext;
function collectPostcssParents(node) {
    const parents = [];
    let current = node.parent;
    while (current) {
        parents.push(current);
        current = current.parent;
    }
    return parents;
}
function getLastWhile(list, check) {
    let validLast;
    while (!validLast && list.length) {
        const current = list.pop();
        if (check(current)) {
            validLast = current;
        }
    }
    return validLast;
}
//# sourceMappingURL=lang-service-context.js.map